@@ -33,14 +33,14 @@ from pykotor.gl.models.predefined import STORE_MDL_DATA, STORE_MDX_DATA, WAYPOIN
     SOUND_MDL_DATA, SOUND_MDX_DATA, CAMERA_MDL_DATA, CAMERA_MDX_DATA, TRIGGER_MDL_DATA, TRIGGER_MDX_DATA, \
     ENCOUNTER_MDL_DATA, ENCOUNTER_MDX_DATA, ENTRY_MDL_DATA, ENTRY_MDX_DATA, EMPTY_MDL_DATA, EMPTY_MDX_DATA
 
-SEARCH_ORDER_2DA = [SearchLocation.CHITIN]
+SEARCH_ORDER_2DA = [SearchLocation.OVERRIDE, SearchLocation.CHITIN]
 SEARCH_ORDER = [SearchLocation.CUSTOM_MODULES, SearchLocation.OVERRIDE, SearchLocation.CHITIN]
 
 
 class Scene:
     SPECIAL_MODELS = ["waypoint", "store", "sound", "camera", "trigger", "encounter"]
 
-    def __init__(self, module: Module, installation: Installation):
+    def __init__(self, installation: Installation, *, module: Optional[Module] = None):
         glEnable(GL_TEXTURE_2D)
         glEnable(GL_DEPTH_TEST)
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
@@ -51,7 +51,7 @@ class Scene:
         self.models: CaseInsensitiveDict[Model] = CaseInsensitiveDict()
         self.objects: List[RenderObject] = []
         self.selection: List[RenderObject] = []
-        self.module: Module = module
+        self.module: Optional[Module] = module
         self.camera: Camera = Camera()
 
         self.textures["NULL"] = Texture.from_color()
@@ -87,6 +87,9 @@ class Scene:
         self.backface_culling: bool = True
 
     def buildCache(self, clearCache: bool = False) -> None:
+        if self.module is None:
+            return
+
         if clearCache:
             self.objects = {}
 
@@ -94,7 +97,7 @@ class Scene:
             for creature in copy(self.git.creatures):
                 if identifier.resname == creature.resref and identifier.restype == ResourceType.UTC:
                     del self.objects[creature]
-            for placeable in copy(self.git.creatures):
+            for placeable in copy(self.git.placeables):
                 if identifier.resname == placeable.resref and identifier.restype == ResourceType.UTP:
                     del self.objects[placeable]
             for door in copy(self.git.doors):
@@ -127,9 +130,6 @@ class Scene:
 
         for door in self.git.doors:
             if door not in self.objects:
-                position = vec3(door.position.x, door.position.y, door.position.z)
-                rotation = vec3(0, 0, door.bearing)
-
                 try:
                     utd = self.module.door(door.resref.get()).resource()
                     model_name = self.table_doors.get_row(utd.appearance_id).get_string("modelname")
@@ -137,16 +137,13 @@ class Scene:
                     # If failed to load creature models, use an empty model instead
                     model_name = "empty"
 
-                self.objects[door] = RenderObject(model_name, position, rotation, data=door)
-            else:
-                self.objects[door].set_position(door.position.x, door.position.y, door.position.z)
-                self.objects[door].set_rotation(0, 0, door.bearing)
+                self.objects[door] = RenderObject(model_name, vec3(), vec3(), data=door)
+
+            self.objects[door].set_position(door.position.x, door.position.y, door.position.z)
+            self.objects[door].set_rotation(0, 0, door.bearing)
 
         for placeable in self.git.placeables:
             if placeable not in self.objects:
-                position = vec3(placeable.position.x, placeable.position.y, placeable.position.z)
-                rotation = vec3(0, 0, placeable.bearing)
-
                 try:
                     utp = self.module.placeable(placeable.resref.get()).resource()
                     model_name = self.table_placeables.get_row(utp.appearance_id).get_string("modelname")
@@ -154,20 +151,17 @@ class Scene:
                     # If failed to load creature models, use an empty model instead
                     model_name = "empty"
 
-                self.objects[placeable] = RenderObject(model_name, position, rotation, data=placeable)
-            else:
-                self.objects[placeable].set_position(placeable.position.x, placeable.position.y, placeable.position.z)
-                self.objects[placeable].set_rotation(0, 0, placeable.bearing)
+                self.objects[placeable] = RenderObject(model_name, vec3(), vec3(), data=placeable)
+
+            self.objects[placeable].set_position(placeable.position.x, placeable.position.y, placeable.position.z)
+            self.objects[placeable].set_rotation(0, 0, placeable.bearing)
 
         for creature in self.git.creatures:
             if creature not in self.objects:
-                position = vec3(creature.position.x, creature.position.y, creature.position.z)
-                rotation = vec3(0, 0, creature.bearing)
-
                 try:
                     utc = self.module.creature(creature.resref.get()).resource()
                     body_model = self.table_creatures.get_row(utc.appearance_id).get_string("race")
-                    obj = RenderObject(body_model, position, rotation, data=creature)
+                    obj = RenderObject(body_model, vec3(), vec3(), data=creature)
 
                     head_str = self.table_creatures.get_row(utc.appearance_id).get_string("normalhead")
                     if head_str:
@@ -178,80 +172,67 @@ class Scene:
                         obj.children.append(head_obj)
                 except Exception:
                     # If failed to load creature models, use an empty model instead
-                    obj = RenderObject("empty", position, rotation, data=creature)
+                    obj = RenderObject("empty", vec3(), vec3(), data=creature)
 
                 self.objects[creature] = obj
-            else:
-                self.objects[creature].set_position(creature.position.x, creature.position.y, creature.position.z)
-                self.objects[creature].set_rotation(0, 0, creature.bearing)
+
+            self.objects[creature].set_position(creature.position.x, creature.position.y, creature.position.z)
+            self.objects[creature].set_rotation(0, 0, creature.bearing)
 
         for waypoint in self.git.waypoints:
             if waypoint not in self.objects:
-                position = vec3(waypoint.position.x, waypoint.position.y, waypoint.position.z)
-                rotation = vec3(0, 0, waypoint.bearing)
-                obj = RenderObject("waypoint", position, rotation, data=waypoint)
+                obj = RenderObject("waypoint", vec3(), vec3(), data=waypoint)
                 self.objects[waypoint] = obj
-            else:
-                self.objects[waypoint].set_position(waypoint.position.x, waypoint.position.y, waypoint.position.z)
-                self.objects[waypoint].set_rotation(0, 0, waypoint.bearing)
+
+            self.objects[waypoint].set_position(waypoint.position.x, waypoint.position.y, waypoint.position.z)
+            self.objects[waypoint].set_rotation(0, 0, waypoint.bearing)
 
         for store in self.git.stores:
             if store not in self.objects:
-                position = vec3(store.position.x, store.position.y, store.position.z)
-                rotation = vec3(0, 0, store.bearing)
-                obj = RenderObject("store", position, rotation, data=store)
+                obj = RenderObject("store", vec3(), vec3(), data=store)
                 self.objects[store] = obj
-            else:
-                self.objects[store].set_position(store.position.x, store.position.y, store.position.z)
-                self.objects[store].set_rotation(0, 0, store.bearing)
+
+            self.objects[store].set_position(store.position.x, store.position.y, store.position.z)
+            self.objects[store].set_rotation(0, 0, store.bearing)
 
         for sound in self.git.sounds:
             if sound not in self.objects:
-                genBoundary = lambda: Empty(self)
                 with suppress(Exception):
                     uts = self.module.sound(sound.resref.get()).resource()
-                    genBoundary = lambda: Boundary.from_circle(self, uts.max_distance)
+                    genBoundary = lambda boundary=Boundary.from_circle(self, uts.max_distance): boundary
 
-                position = vec3(sound.position.x, sound.position.y, sound.position.z)
-                rotation = vec3(0, 0, 0)
-                obj = RenderObject("sound", position, rotation, data=sound, genBoundary=genBoundary)
+                obj = RenderObject("sound", vec3(), vec3(), data=sound, genBoundary=genBoundary)
                 self.objects[sound] = obj
-            else:
-                self.objects[sound].set_position(sound.position.x, sound.position.y, sound.position.z)
-                self.objects[sound].set_rotation(0, 0, 0)
+
+            self.objects[sound].set_position(sound.position.x, sound.position.y, sound.position.z)
+            self.objects[sound].set_rotation(0, 0, 0)
 
         for encounter in self.git.encounters:
             if encounter not in self.objects:
-                position = vec3(encounter.position.x, encounter.position.y, encounter.position.z)
-                rotation = vec3(0, 0, 0)
-                genBoundary = lambda: Boundary(self, encounter.geometry.points)
-                obj = RenderObject("encounter", position, rotation, data=encounter, genBoundary=genBoundary)
+                genBoundary = lambda boundary=Boundary(self, encounter.geometry.points): boundary
+                obj = RenderObject("encounter", vec3(), vec3(), data=encounter, genBoundary=genBoundary)
                 self.objects[encounter] = obj
-            else:
-                self.objects[encounter].set_position(encounter.position.x, encounter.position.y, encounter.position.z)
-                self.objects[encounter].set_rotation(0, 0, 0)
+
+            self.objects[encounter].set_position(encounter.position.x, encounter.position.y, encounter.position.z)
+            self.objects[encounter].set_rotation(0, 0, 0)
 
         for trigger in self.git.triggers:
             if trigger not in self.objects:
-                position = vec3(trigger.position.x, trigger.position.y, trigger.position.z)
-                rotation = vec3(0, 0, 0)
-                genBoundary = lambda: Boundary(self, trigger.geometry.points)
-                obj = RenderObject("trigger", position, rotation, data=trigger, genBoundary=genBoundary)
+                genBoundary = lambda boundary=Boundary(self, trigger.geometry.points): boundary
+                obj = RenderObject("trigger", vec3(), vec3(), data=trigger, genBoundary=genBoundary)
                 self.objects[trigger] = obj
-            else:
-                self.objects[trigger].set_position(trigger.position.x, trigger.position.y, trigger.position.z)
-                self.objects[trigger].set_rotation(0, 0, 0)
+
+            self.objects[trigger].set_position(trigger.position.x, trigger.position.y, trigger.position.z)
+            self.objects[trigger].set_rotation(0, 0, 0)
 
         for camera in self.git.cameras:
             if camera not in self.objects:
-                position = vec3(camera.position.x, camera.position.y, camera.position.z)
-                rotation = glm.eulerAngles(quat(camera.orientation.x, camera.orientation.y, camera.orientation.z, camera.orientation.w))
-                obj = RenderObject("camera", position, rotation, data=camera)
+                obj = RenderObject("camera", vec3(), vec3(), data=camera)
                 self.objects[camera] = obj
-            else:
-                self.objects[camera].set_position(camera.position.x, camera.position.y, camera.position.z)
-                euler = glm.eulerAngles(quat(camera.orientation.x, camera.orientation.y, camera.orientation.z, camera.orientation.w))
-                self.objects[camera].set_rotation(euler.x, euler.y, euler.z)
+
+            self.objects[camera].set_position(camera.position.x, camera.position.y, camera.position.z)
+            euler = glm.eulerAngles(quat(camera.orientation.w, camera.orientation.x, camera.orientation.y, camera.orientation.z))
+            self.objects[camera].set_rotation(euler.y, euler.z-math.pi/2, -euler.x+math.pi/2)
 
         # Detect if GIT still exists; if they do not then remove them from the render list
         for obj in copy(self.objects):
@@ -348,6 +329,9 @@ class Scene:
         glClearColor(1.0, 1.0, 1.0, 1.0)
         glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
 
+        if self.backface_culling:
+            glEnable(GL_CULL_FACE)
+
         self.picker_shader.use()
         self.picker_shader.set_matrix4("view", self.camera.view())
         self.picker_shader.set_matrix4("projection", self.camera.projection())
@@ -428,19 +412,34 @@ class Scene:
                 mdl_data = EMPTY_MDL_DATA
                 mdx_data = EMPTY_MDX_DATA
             else:
-                mdl_data = self.installation.resource(name, ResourceType.MDL, SEARCH_ORDER, capsules=self.module.capsules()).data
-                mdx_data = self.installation.resource(name, ResourceType.MDX, SEARCH_ORDER, capsules=self.module.capsules()).data
+                mdl_search = self.installation.resource(name, ResourceType.MDL, SEARCH_ORDER, capsules=self.module.capsules())
+                mdx_search = self.installation.resource(name, ResourceType.MDX, SEARCH_ORDER, capsules=self.module.capsules())
+                if mdl_search and mdx_search:
+                    mdl_data = mdl_search.data
+                    mdx_data = mdx_search.data
+                else:
+                    mdl_data = EMPTY_MDL_DATA
+                    mdx_data = EMPTY_MDX_DATA
 
             # model = gl_load_mdl(self, BinaryReader.from_bytes(mdl_data, 12), BinaryReader.from_bytes(mdx_data))
-            model = gl_load_stitched_model(self, BinaryReader.from_bytes(mdl_data, 12), BinaryReader.from_bytes(mdx_data))
+            try:
+                model = gl_load_stitched_model(self, BinaryReader.from_bytes(mdl_data, 12), BinaryReader.from_bytes(mdx_data))
+            except Exception:
+                model = gl_load_stitched_model(self, BinaryReader.from_bytes(EMPTY_MDL_DATA, 12), BinaryReader.from_bytes(EMPTY_MDX_DATA))
+
             self.models[name] = model
         return self.models[name]
 
     def jumpToEntryLocation(self) -> None:
-        point = self.module.info().resource().entry_position
-        self.camera.x = point.x
-        self.camera.y = point.y
-        self.camera.z = point.z + 1.8
+        if self.module is None:
+            self.camera.x = 0
+            self.camera.y = 0
+            self.camera.z = 0
+        else:
+            point = self.module.info().resource().entry_position
+            self.camera.x = point.x
+            self.camera.y = point.y
+            self.camera.z = point.z + 1.8
 
 
 class RenderObject:
@@ -532,80 +531,29 @@ class Camera:
         self.x: float = 40.0
         self.y: float = 130.0
         self.z: float = 0.5
-        self.pitch: float = math.pi /2
+        self.pitch: float = math.pi / 2
         self.yaw: float = 0.0
+        self.distance: float = 10.0
         self.fov: float = 90.0
         self.aspect: float = 16 / 9
 
     def view(self) -> mat4:
         up = vec3(0, 0, 1)
-        camera = glm.translate(mat4(), vec3(self.x, self.y, self.z))
-        camera = glm.rotate(camera, self.yaw, up)
         pitch = glm.vec3(1, 0, 0)
-        camera = glm.rotate(camera, self.pitch, pitch)
+
+        x, y, z = self.x, self.y, self.z
+        x += math.cos(self.yaw) * math.cos(self.pitch - math.pi/2) * self.distance
+        y += math.sin(self.yaw) * math.cos(self.pitch - math.pi/2) * self.distance
+        z += math.sin(self.pitch - math.pi/2) * self.distance
+
+        camera = glm.translate(mat4(), vec3(x, y, z))
+        camera = glm.rotate(camera, self.yaw + math.pi/2, up)
+        camera = glm.rotate(camera, math.pi - self.pitch, pitch)
         view = glm.inverse(camera)
         return view
 
     def projection(self) -> mat4:
-        return glm.perspective(90, self.aspect, 0.1, 5000)
-
-    def translate(self, translation: vec3) -> None:
-        self.x += translation.x
-        self.y += translation.y
-        self.z += translation.z
-
-    def rotate(self, yaw: float, pitch: float):
-        self.pitch += pitch
-        self.yaw += yaw
-
-        if self.pitch > math.pi:
-            self.pitch = math.pi
-        elif self.pitch < 0:
-            self.pitch = 0
-
-    def forward(self, ignoreZ: bool = True) -> vec3:
-        eye_x = -math.cos(self.yaw) * math.cos(self.pitch - math.pi/2)
-        eye_y = math.sin(self.yaw) * math.cos(self.pitch - math.pi/2)
-        eye_z = 0 if ignoreZ else math.sin(self.pitch - math.pi/2)
-        return -glm.normalize(vec3(eye_y, eye_x, eye_z))
-
-    def sideward(self, ignoreZ: bool = True) -> vec3:
-        eye_x = math.cos(self.yaw) * math.cos(self.pitch - math.pi/2)
-        eye_y = math.sin(self.yaw) * math.cos(self.pitch - math.pi/2)
-        eye_z = 0 if ignoreZ else math.sin(self.pitch - math.pi/2)
-        return glm.normalize(-vec3(eye_x, eye_y, eye_z))
-
-    def upward(self, ignoreXY: bool = True) -> vec3:
-        if not ignoreXY:
-            raise NotImplementedError
-        eye_y = 0 if ignoreXY else math.cos(self.pitch) * math.sin(self.yaw)
-        eye_x = 0 if ignoreXY else math.cos(self.yaw) * math.cos(self.pitch)
-        eye_z = 1
-        return glm.normalize(vec3(eye_y, eye_x, eye_z))
-
-
-class FocusedCamera:
-    def __init__(self):
-        self.x: float = 40.0
-        self.y: float = 130.0
-        self.z: float = 0.5
-        self.pitch: float = math.pi / 2
-        self.yaw: float = 0.0
-        self.distance: float = 2.0
-        self.fov: float = 90.0
-        self.aspect: float = 16 / 9
-
-    def view(self) -> mat4:
-        eye_x = self.x + math.cos(self.yaw) * math.sin(self.pitch)
-        eye_y = self.y + math.sin(self.yaw) * math.sin(self.pitch)
-        eye_z = self.z + math.cos(self.pitch)
-
-        eye = vec3(eye_x, eye_y, eye_z)
-        centre = vec3(self.x, self.y, self.z)
-        return glm.lookAt(eye, centre, vec3(0, 0, 1))
-
-    def projection(self) -> mat4:
-        return glm.perspective(90, 16 / 9, 0.1, 5000)
+        return glm.perspective(self.fov, self.aspect, 0.1, 5000)
 
     def translate(self, translation: vec3) -> None:
         self.x += translation.x
@@ -622,21 +570,26 @@ class FocusedCamera:
             self.pitch = 0.001
 
     def forward(self, ignoreZ: bool = True) -> vec3:
-        eye_x = -math.cos(self.yaw) * math.sin(self.pitch)
-        eye_y = -math.sin(self.yaw) * math.sin(self.pitch)
-        eye_z = 0 if ignoreZ else math.sin(self.pitch)
-        return glm.normalize(vec3(eye_x, eye_y, eye_z))
+        eye_x = math.cos(self.yaw) * math.cos(self.pitch - math.pi / 2)
+        eye_y = math.sin(self.yaw) * math.cos(self.pitch - math.pi / 2)
+        eye_z = 0 if ignoreZ else math.sin(self.pitch - math.pi / 2)
+        return glm.normalize(-vec3(eye_x, eye_y, eye_z))
 
     def sideward(self, ignoreZ: bool = True) -> vec3:
-        eye_x = math.sin(self.yaw) * math.sin(self.pitch)
-        eye_y = -math.cos(self.yaw) * math.sin(self.pitch)
-        eye_z = 0 if ignoreZ else math.sin(self.pitch)
-        return glm.normalize(vec3(eye_x, eye_y, eye_z))
+        return glm.normalize(glm.cross(self.forward(ignoreZ), vec3(0.0, 0.0, 1.0)))
 
     def upward(self, ignoreXY: bool = True) -> vec3:
         if not ignoreXY:
-            raise NotImplementedError
-        eye_y = 0 if ignoreXY else math.cos(self.pitch) * math.sin(self.yaw)
-        eye_x = 0 if ignoreXY else math.cos(self.yaw) * math.cos(self.pitch)
-        eye_z = 1
-        return glm.normalize(vec3(eye_y, eye_x, eye_z))
+            forward = self.forward(False)
+            sideward = self.sideward(False)
+            cross = glm.cross(forward, sideward)
+            return glm.normalize(cross)
+        else:
+            return glm.normalize(vec3(0, 0, 1))
+
+    def truePosition(self) -> vec3:
+        x, y, z = self.x, self.y, self.z
+        x += math.cos(self.yaw) * math.cos(self.pitch - math.pi / 2) * self.distance
+        y += math.sin(self.yaw) * math.cos(self.pitch - math.pi / 2) * self.distance
+        z += math.sin(self.pitch - math.pi / 2) * self.distance
+        return vec3(x, y, z)
\ No newline at end of file
